/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.providers.jca;

import javax.wsdl.*;

/**
 * This class is the top level superclass for all JCA WSIF messages.  It is extended by the connector specific messages, 
 * and implements the javax.resource.cci.Record interface.
 * 
 * @author Michael Beisiegel
 * @author Piotr Przybylski <piotrp@ca.ibm.com>
 */

public class WSIFMessage_JCA extends org.apache.wsif.base.WSIFDefaultMessage implements javax.resource.cci.Record {

	private static final long serialVersionUID = 1L;
	public static final int INPUT_MESSAGE = 1;
	public static final int OUTPUT_MESSAGE = 2;
	public static final int FAULT_MESSAGE = 3;
	
	protected String fieldRecordName = null;
	protected String fieldRecordShortDescription = null;

	protected Definition fieldDefinition = null;
	protected Binding fieldBinding = null;
	protected String fieldOperationName = null;
	protected String fieldInputName = null; 
	protected String fieldOutputName = null;
	protected int fieldMessageType = 0;
	protected javax.resource.cci.InteractionSpec fieldInteractionSpec = null;	
		
	/**
	 * WSIFMessage_JCA Constructor
	 * @param aDefinition A Definition containing the operation to execute.
	 * @param aBinding A service binding
	 * @param aOperationName The name of the operation
	 * @param aInputName Input name
	 * @param aOutputName Output name
	 * @param aMessageType Determines what kind of message is created, input, output or fault. 
	 */
	public WSIFMessage_JCA(Definition aDefinition, Binding aBinding, String aOperationName, String aInputName, String aOutputName, int aMessageType) {
		super();
		this.fieldDefinition = aDefinition;
		this.fieldBinding = aBinding;
		this.fieldOperationName = aOperationName;
		this.fieldInputName = aInputName;
		this.fieldOutputName = aOutputName;
		this.fieldMessageType = aMessageType;
	}

	/**
	 * Gets the recordName.
	 * @return Returns a String
	 */
	public String getRecordName() {
		return fieldRecordName;
	}

	/*
	 * @see Record#setRecordName(String)
	 */
	public void setRecordName(String name) {
		this.fieldRecordName = name;
	}

	/*
	 * @see Record#setRecordShortDescription(String)
	 */
	public void setRecordShortDescription(String desription) {
		this.fieldRecordShortDescription = desription;
	}

	/**
	 * Gets the recordShortDescription.
	 * @return Returns a String
	 */
	public String getRecordShortDescription() {
		return fieldRecordShortDescription;
	}

	/**
	 * Method not supported.
	 */
	public Object clone()throws CloneNotSupportedException{
		throw new CloneNotSupportedException();
	}
	/**
	 * Sets the interactionSpec.
	 * @param interactionSpec The interactionSpec to set
	 */
	public void setInteractionSpec(javax.resource.cci.InteractionSpec interactionSpec) {
		fieldInteractionSpec = interactionSpec;
	}

}