/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.wsif.providers.jca.toolplugin;
/**
 * This class represents an XSD definition that may be required for an Imported Service. The Resource Adapter
 * does not need to implement this class if it returns XSD types inlined within Definition.
 *  @author Hesham Fahmy <hfahmy@ca.ibm.com>
 *  @author Piotr Przybylski <piotrp@ca.ibm.com>
 */
public class ImportXSD implements java.io.Serializable {

	private static final long serialVersionUID = 1L;
	private String fieldNamespace = null;
	private String fieldLocation = null;
	private String fieldSource = null;

	/**
	 * Returns the namespace that is associated with the XSD definition
	 * @return Returns a String representation of the namespace
	 */
	public String getNamespace() {
		return fieldNamespace;
	}
	/**
	 * Sets the namespace that is associated with the XSD definition
	 * @param <code>namespace</code> The namespace to set
	 */
	public void setNamespace(String namespace) {
		this.fieldNamespace = namespace;
	}
	/**
	 * Returns a String representation of the relative path where the XSD should be saved.
	 * The path is relative to the location of the WSDL document that represents the imported service.
	 * @return Returns a String
	 */
	public String getLocation() {
		return fieldLocation;
	}
	/**
	 * Sets the relative path where the XSD should be saved.
	 * @param <code>location</code> The location to set
	 */
	public void setLocation(String location) {
		this.fieldLocation = location;
	}
	/**
	 * Gets the contents of the XSD definiton
	 * @return Returns a String representation of the contents of the XSD
	 */
	public String getSource() {
		return fieldSource;
	}
	/**
	 * Sets the contents of the XSD definiton.
	 * @param <code>source</code> The XSD source to set
	 */
	public void setSource(String source) {
		this.fieldSource = source;
	}
}

