/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package mime;

import java.awt.Image;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.swing.ImageIcon;

import util.TestUtilities;

/**
 * Mime service used by MimeTest 
 * @author Mark Whitlock
 */

public class MimeImpl {

    public String dataHandlerToString(DataHandler dh) {
        try {
            InputStream is = dh.getInputStream();
            byte[] bBuff = new byte[is.available()];
            is.read(bBuff);
            String sBuff = new String(bBuff);
            return sBuff;
        } catch (IOException ioe) {
            ioe.printStackTrace();
            return null;
        }
    }

    public DataHandler stringToDataHandler(String buff) {
        try {
            FileDataSource fds = getTempFile();
            fds.getOutputStream().write(buff.getBytes());
            DataHandler dh = new DataHandler(fds);
            return dh;
        } catch (IOException ioe) {
            ioe.printStackTrace();
            return null;
        }
    }

    public String plainTextToString(DataHandler ds) {
        try {
            InputStream is = ds.getInputStream();
            byte[] bBuff = new byte[is.available()];
            is.read(bBuff);
            String sBuff = new String(bBuff);
            return sBuff;
        } catch (IOException ioe) {
            ioe.printStackTrace();
            return null;
        }
    }

    public DataHandler stringToPlainText(String buff) {
        try {
            FileDataSource fds = getTempFile();
            fds.getOutputStream().write(buff.getBytes());
            return new DataHandler(fds);
        } catch (IOException ioe) {
            ioe.printStackTrace();
            return null;
        }
    }

    public DataHandler bounceImage(DataHandler ds) {
        try {
            InputStream is = ds.getInputStream();
            byte[] bBuff = new byte[is.available()];
            is.read(bBuff);

//            This commented out code displays the image
//            to check that it is correct.
//
//            Image im = new ImageIcon(bBuff).getImage();
//            WSIFFrame.display(im, "Backend image");
//            int t = 0;
//            try {
//                t = Integer.parseInt(
//                    TestUtilities.getWsifProperty("wsif.displaytime"));
//            } finally {
//                if (t <= 0)
//                    t = 2000;
//            }
//            Thread.sleep(t);

            FileDataSource fds = getTempFile();
            fds.getOutputStream().write(bBuff);
            return new DataHandler(fds);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        } finally {
            WSIFFrame.close();
        }
    }

    public DataHandler bounceImage2(DataHandler ds) {
        return bounceImage(ds);
    }

    //TODO: See bugzilla bug 15837
    public DataHandler bounceImage4(boolean b, DataHandler ds) {
//    	return bounceImage4(ds, b);
//    }
//
//    public DataHandler bounceImage4(DataHandler ds, boolean b) {
        if (b && (ds != null))
            return bounceImage(ds);
        else
            return null;
    }

    public String orMultiMimeParts(DataHandler dh) {
    	return dh.getContentType();
    }

    public String andMultiMimeParts(DataHandler dh1, DataHandler dh2) {
        try {
            InputStream is1 = dh1.getInputStream();
            byte[] bBuff1 = new byte[is1.available()];
            is1.read(bBuff1);
            StringBuffer sBuff = new StringBuffer(new String(bBuff1));
            
            InputStream is2 = dh2.getInputStream();
            byte[] bBuff2 = new byte[is2.available()];
            is2.read(bBuff2);
            sBuff.append(new String(bBuff2));
            
            return sBuff.toString();
        } catch (IOException ioe) {
            ioe.printStackTrace();
            return null;
        }
    }

    // MUST IMPLEMENT ???    
    // multiOutMimeParts and multiInoutMimeParts are only 
    // invoked through the stubless interface

    public String noContent(String s) {
    	return s;
    }

    public String typeStar(DataHandler dh) {
    	return dataHandlerToString(dh);
    }

    public String soapBodyParts1(boolean shouldBounce, DataHandler dh) {
        return "1";
    }

    public String soapBodyParts2(DataHandler dh) {
        return "2";
    }

    public String soapBodyParts3(boolean shouldBounce) {
        return "3";
    }

    public String soapBodyParts4() {
        return "4";
    }

    public String arrayOfBinary(DataHandler dh) {
    	return dataHandlerToString(dh);
    }

    public DataHandler optionalSoapBody(DataHandler ds) {
    	return bounceImage(ds);
    }

    public String mixMimeParts(
        String pos1,
        DataHandler dh1,
        String pos2,
        DataHandler dh2,
        String pos3)
    {
        return pos1
            + dataHandlerToString(dh1)
            + pos2
            + dataHandlerToString(dh2)
            + pos3;
    }

    /* ******************* ERRORS *********************** */
    
    public String badNoPart(DataHandler dh) {
    	return dataHandlerToString(dh);
    }

    public String badPart(DataHandler dh) {
    	return dataHandlerToString(dh);
    }
    
    public String badNested(DataHandler dh) {
    	return dataHandlerToString(dh);
    }

    public String badMixSoapMime(DataHandler dh) {
    	return dataHandlerToString(dh);
    }

    public String badMultipleSoapBodies(DataHandler dh) {
    	return dataHandlerToString(dh);
    }

    public String badSoapBodyType(DataHandler dh) {
    	return dataHandlerToString(dh);
    }

    /* ***************** UTILITY METHODS *************** */
    
    private FileDataSource getTempFile() throws IOException {
        File f = File.createTempFile("WSIFMimeTest", "txt");
        f.deleteOnExit();
        return new FileDataSource(f);
    }
}
